import pathlib
import subprocess

import structlog

from .parse_repo_url import RepoURL


def get_destination_path_for_repo(
    repo_url: RepoURL, prefix_path: pathlib.Path
) -> pathlib.Path:
    return prefix_path / repo_url.host / repo_url.group / repo_url.project


def clone(
    repo_url: RepoURL,
    no_act: bool,
    fetch: bool,
    prefix_path: pathlib.Path,
) -> None:
    log = structlog.get_logger()
    destination_path = get_destination_path_for_repo(repo_url, prefix_path)
    if destination_path.exists() and fetch and repo_url.tool == "git":
        cmd = ["git", "fetch", "--all", "--prune", "--recurse-submodules"]
        log.info("run", cmd=cmd, cwd=destination_path)
        if not no_act:
            subprocess.run(cmd, check=True, cwd=str(destination_path))
        return

    path = str(prefix_path / repo_url.host / repo_url.group / repo_url.project)
    if repo_url.tool == "hg":
        cmd = ["hg", "clone", repo_url.url, path]
    else:
        cmd = [repo_url.tool, "clone", "--recurse-submodules", repo_url.url, path]
    log.info("run", cmd=cmd)
    if not no_act:
        subprocess.run(cmd, check=True)



"""Parse git/hd/other repo urls into a neat usable format

Tries to handle all the exotic repos, suitable for cloning.

"""

import dataclasses
import pathlib
import re
import typing


@dataclasses.dataclass
class RepoURL:
    url: str
    tool: str
    host: str
    group: str
    project: str


def parse_url(url: str) -> typing.Optional[RepoURL]:
    """Parses the given repo url"""
    if url.startswith("file://") or str(pathlib.Path(url).expanduser()).startswith("/"):
        return RepoURL(url, "git", "localhost", "file", pathlib.Path(url).name)

    m = re.match(
        r"(?P<tool>[^:@]+)(://|@)(?P<tool_user>hg@|git@){0,1}(?P<host>[^/:]+):{0,1}(?P<port>[0-9]+){0,1}(/|:)((?P<group>[^/]+)/){0,1}(?P<project>.+)",
        url,
    )
    if m is None:
        return None
    matched = m.groupdict()
    tool = matched["tool"]
    tool_user = matched.get("tool_user", None)
    if tool == "ssh":
        if tool_user == "hg@":
            tool = "hg"
        else:
            tool = "git"
    if tool in ("keybase", "https", "man"):
        tool = "git"
    host = matched["host"]
    if host.startswith("hg."):
        tool = "hg"
    port = matched.get("port", None)
    if port is not None:
        host = "{}:{}".format(host, port)
    if matched["tool"] == "keybase":
        host = "keybase"
    group = matched["group"]
    if group is None:
        group = ""
    project = re.split(r"\.git$", matched["project"])[0]
    if project.endswith("/"):
        project = project[:-1]
    return RepoURL(url, tool, host, group, project)



#!/usr/bin/env python3

import logging
import pathlib
import typing

import structlog
import structlog.contextvars
import structlog.processors
import structlog.types
import typer

from . import parse_repo_url
from .clone import clone

app = typer.Typer()


@app.command()
def main(
    url: str = typer.Argument(..., help="Repo URl to clone"),
    prefix: str = typer.Option(
        "~/src", help="Prefix to clone to. Will clone to `{prefix}/{host}/{org}/{repo}`"
    ),
    verbose: bool = typer.Option(
        False, "--verbose/--no-verbose", "-v", help="Enable more verbose log output"
    ),
    debug: bool = typer.Option(
        False, "--debug/--no-debug", "-d", help="Enable more debug log output"
    ),
    json: bool = typer.Option(
        False,
        "--json/--no-json",
        "-j",
        envvar="CLONE_REPO_JSON_LOGS",
        help="Use JSON for log output",
    ),
    no_act: bool = typer.Option(
        False, "--no-act/--act", "-n", help="Simulate the clone"
    ),
    fetch: bool = typer.Option(
        False,
        "--fetch/--no-fetch",
        "-f",
        help="If the destination exists use a git fetch instead",
    ),
) -> None:
    """Clone the given repo URL to a common prefix"""
    prefix_path = pathlib.Path(prefix).expanduser().resolve()

    processors: typing.List[structlog.types.Processor] = (
        [
            structlog.contextvars.merge_contextvars,
            structlog.processors.add_log_level,
            structlog.processors.StackInfoRenderer(),
            structlog.processors.format_exc_info,
            structlog.processors.TimeStamper(fmt="iso"),
            structlog.processors.JSONRenderer(),
        ]
        if json
        else [
            structlog.contextvars.merge_contextvars,
            structlog.processors.add_log_level,
            structlog.processors.StackInfoRenderer(),
            structlog.processors.TimeStamper(fmt="iso"),
            structlog.dev.ConsoleRenderer(),
        ]
    )

    structlog.configure(
        cache_logger_on_first_use=True,
        wrapper_class=structlog.make_filtering_bound_logger(
            logging.INFO if verbose else (logging.DEBUG if debug else logging.WARNING)
        ),
        processors=processors,
    )

    structlog.contextvars.reset_contextvars()
    structlog.contextvars.bind_contextvars(
        url=url,
        prefix=prefix_path,
        verbose=verbose,
        debug=debug,
        json=json,
        no_act=no_act,
        fetch=fetch,
    )

    log = structlog.get_logger()

    log.debug("config", config=structlog.get_config())

    try:
        repo_url = parse_repo_url.parse_url(url)
        if repo_url is None:
            raise ValueError(f"Couldn't parse {url}")

        clone(
            repo_url=repo_url,
            no_act=no_act,
            fetch=fetch,
            prefix_path=prefix_path,
        )
    except Exception as e:
        log.fatal(f"Error: {e}", exc_info=debug or json)
        raise typer.Exit(code=1)


if __name__ == "__main__":
    app()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['clone_repo']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1,<9.0',
 'rich>=12.0,<13.0',
 'structlog>=21.5,<22.0',
 'typer>=0.4,<0.5']

entry_points = \
{'console_scripts': ['clone-repo = clone_repo.main:app']}

setup_kwargs = {
    'name': 'clone-repo',
    'version': '0.2.0',
    'description': 'CLI tool to clone repos easily',
    'long_description': "# clone-repo: Easily clone repos\n\n- [Home](https://github.com/micktwomey/clone-repo)\n- [PyPI](https://pypi.org/project/clone-repo/)\n\n## What is this?\n\nFor years I've been using a noddy script which clones many types of repos into a fixed location which is generated based on the repo itself. Similar to how go packages are cloned into go/src I typically clone my stuff into ~/src.\n\ne.g. `clone-repo git@github.com:micktwomey/clone-repo.git` will clone using git to `~/src/github.com/micktwomey/clone-repo`.\n\nThis allows me to clone many different repos without worrying about stepping on each other. It also makes it easy to see where stuff comes from (e.g. `rm -rf ~/src/old.git.example.com` will wipe out clones from git server I don't use any more).\n\nInstall via `pip install clone-repo`.\n\nInstall with [pipx](https://pypa.github.io/pipx/) using `pipx install clone-repo` to make it available as a CLI tool everywhere.\n\nSupports:\n\n- `/path/to/repo`\n  - `git clone` to `~/src/localhost/file/{repo}`\n- `file:///path/to/repo`\n  - `git clone` to `~/src/localhost/file/{repo}`\n- `git@example.com:org/repo.git`\n  - `git clone` to `~/src/{host}/{org}/{repo}`\n- `https://github.com/org/repo.git`\n  - `git clone` to `~/src/github.com/{org}/{repo}`\n- `https://gitlab.example.com/org/repo.git`\n  - `git clone` to `~/src/{host}/{org}/{repo}`\n- `https://hg.mozilla.org/mozilla-central/`\n  - `hg clone` to `~/src/hg.mozilla.org/{org}/{repo}`\n- `https://hg.sr.ht/~org/repo`\n  - `hg clone` to `~/src/hg.sr.ht/{org}/{repo}`\n- `keybase://team/org/repo`\n  - `git clone` to `~/src/keybase/{org}/{repo}`\n- `man@man.sr.ht:~org/repo`\n  - `git clone` to `~/src/man.sr.ht/{org}/{repo}`\n- `ssh://git@example.com:7999/somegroup/myrepo.git`\n  - `git clone` to `~/src/{host}/{org}/{repo}`\n- `ssh://hg@bitbucket.org/org/repo`\n  - `hg clone` to `~/src/{host}/{org}/{repo}`\n\nFor `https://` URLs the default is git but some will behave differently based on the domain.\n\n# Development\n\nIf you want to quickly develop you can use [poetry](https://python-poetry.org) and [pytest](https://pytest.org/):\n\n1. `poetry install`\n2. `pytest -vv`\n\nIf you want to test across all supported Python versions you can install them via [asdf](https://asdf-vm.com) and then use [nox](https://nox.thea.codes):\n\n1. `asdf install`\n2. `poetry install`\n3. `nox`\n\nIf you want to run [pre-commit](https://pre-commit.com) hooks before committing:\n\n1. `poetry install`\n2. `pre-commit install`\n\nFinally, there is a [just](https://github.com/casey/just) justfile to run some commands.\n",
    'author': 'Michael Twomey',
    'author_email': 'mick@twomeylee.name',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/micktwomey/clone-repo',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import nox


@nox.session()
def black(session: nox.Session) -> None:
    session.install(".")
    session.install("black")
    session.run("black", "--check", ".")


@nox.session()
def isort(session: nox.Session) -> None:
    session.install(".")
    session.install("isort")
    session.run("isort", "--check", ".")


@nox.session(python=["3.7", "3.8", "3.9", "3.10"])
def mypy(session: nox.Session) -> None:
    session.install(".")
    session.install("mypy", "nox", "pytest")
    session.run("mypy", ".")


@nox.session(python=["3.7", "3.8", "3.9", "3.10"])
def tests(session: nox.Session) -> None:
    session.install(".")
    session.install("pytest")
    session.run("pytest", "-vv", ".")
